from UniParse import FileParser
from resume_parser import ResumeParser

parser = FileParser('sample.pdf')
parser = ResumeParser('sample.pdf')
data = parser.get_extracted_data()

print("Resume Data:")
print(data)




import re
import spacy
from UniParse import FileParser


try:
    nlp = spacy.load('en_core_web_sm')
except:
    import subprocess
    subprocess.run(["python", "-m", "spacy", "download", "en_core_web_sm"])
    nlp = spacy.load('en_core_web_sm')

# self.nlp = nlp


class ResumeParser:
    """
    A class to parse resumes and extract structured information.
    """

    def __init__(self, file_path) -> None:
        self.file_path = file_path
        self.text = ""
        self.data = {}
        self.nlp = nlp

    def extract_info(self):
        """
        Extracts text from the resume file using FileParser.
        """
        parser = FileParser(self.file_path)
        self.text = parser.parse()
        if not self.text:
            raise ValueError("Failed to extract text from the resume.")

    def parse(self):
        """
        Parses the resume and extracts information.
        """
        self.extract_info()
        doc = self.nlp(self.text)
        self.data["name"] = self.extract_name(doc)
        self.data["email"] = self.extract_email()
        self.data["mobile_number"] = self.extract_mobile_number()
        self.data["skills"] = self.extract_skills()
        self.data["degree"] = self.extract_degree()
        self.data["college_name"] = self.extract_college()
        self.data["total_experience"] = self.extract_experience()
        self.data["is_experienced"] = bool(self.data["total_experience"])
        self.data["no_of_pages"] = self.get_number_of_pages()
        self.data["hobbies"] = self.extract_hobbies()

    def extract_name(self, doc):
        """
        Extracts the name from the resume using NER.
        """
        lines = self.text.strip().split("\n")

        lines = [line.strip() for line in lines if line.strip()]

        possible_names = []
        for i in range(min(10, len(lines))):
            line = lines[i]
            for ent in doc.ents:
                if ent.label_ == "PERSON" and len(ent.text.split()) <= 3:
                    possible_names.append(ent.text)

            tokens = line.split()
            capitalized_tokens = [token for token in tokens if token[0].isupper()]
            if 1 < len(capitalized_tokens) <= 3 and len(tokens) <= 5:
                possible_names.append(" ".join(capitalized_tokens))

        if possible_names:
            return possible_names[0]
        else:
            return None

    def extract_email(self):
        """
        Extracts email addresses using regex.
        """
        email_pattern = r"[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+"
        emails = re.findall(email_pattern, self.text)
        return emails[0] if emails else None

    def extract_mobile_number(self):
        """
        Extracts mobile numbers using regex.
        """
        phone_pattern = r"(\+?\d{1,3}[-.\s]?)?(\d{5}[-.\s]?\d{5})"
        match = re.search(phone_pattern, self.text)
        if match:
            return match.group()
        return None

    def extract_skills(self):
        """
        Extracts skills from the resume.
        """
        # Define a set of skills to look for
        skills = set(
            [
                "python",
                "java",
                "c++",
                "machine learning",
                "data analysis",
                "sql",
                "javascript",
                "aws",
                "react",
                "django",
                "flask",
                "docker",
                "kubernetes",
                "terraform",
                "git",
                "pandas",
                "numpy",
                "matplotlib",
                "fastapi",
                "restful api",
                "celery",
                "airflow",
            ]
        )
        extracted_skills = []
        for skill in skills:
            if re.search(r"\b" + re.escape(skill) + r"\b", self.text, re.IGNORECASE):
                extracted_skills.append(skill)
        return extracted_skills

    def extract_degree(self):
        """
        Extracts degree information.
        """
        degrees = ["Bachelor", "Master", "B.Sc", "M.Sc", "B.Tech", "M.Tech", "PhD"]
        for degree in degrees:
            if re.search(r"\b" + degree + r"\b", self.text, re.IGNORECASE):
                return degree
        return None

    def extract_college(self):
        """
        Extracts college or university name.
        """
        education_section = ""
        education_keywords = [
            "Education",
            "Academic",
            "Qualification",
            "Educational Background",
            "Education and Achievement",
        ]

        for keyword in education_keywords:
            pattern = rf"{keyword}.*"
            match = re.search(pattern, self.text, re.IGNORECASE | re.DOTALL)
            if match:
                education_section = self.text[match.start() :]
                break

        if education_section:
            lines = education_section.split("\n")
            for line in lines:
                if "B.Tech" in line or "Bachelor" in line:
                    return line.strip()

        return None

    def extract_experience(self):
        """
        Extracts total experience in years from the 'Experience' section only.
        """

        from dateparser import parse
        from datetime import datetime
        import re

        date_patterns = [
            # MM/YYYY - MM/YYYY (e.g., 09/2022 - 12/2023)
            r"(?P<start_month>\d{2})\/(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_month>\d{2})\/(?P<end_year>\d{4}|current|Current|present|Present)",

            # Month YYYY - Month YYYY (e.g., September 2022 - December 2023)
            r"(?P<start_month>\w+)\s+(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_month>\w+)\s+(?P<end_year>\d{4}|current|Current|present|Present)",

            # MMM YYYY - MMM YYYY (e.g., Sep 2022 - Dec 2023)
            r"(?P<start_month>[a-zA-Z]{3})\s+(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_month>[a-zA-Z]{3})\s+(?P<end_year>\d{4}|current|Current|present|Present)",

            # MM-YYYY - MM-YYYY (e.g., 09-2022 - 12-2023)
            r"(?P<start_month>\d{2})-(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_month>\d{2})-(?P<end_year>\d{4}|current|Current|present|Present)",

            # YYYY/MM - YYYY/MM (e.g., 2022/09 - 2023/12)
            r"(?P<start_year>\d{4})\/(?P<start_month>\d{2})\s*[-–to]+\s*(?P<end_year>\d{4})\/(?P<end_month>\d{2}|current|Current|present|Present)",

            # YYYY-MM - YYYY-MM (e.g., 2022-09 - 2023-12)
            r"(?P<start_year>\d{4})-(?P<start_month>\d{2})\s*[-–to]+\s*(?P<end_year>\d{4})-(?P<end_month>\d{2}|current|Current|present|Present)",

            # Month YYYY - Present (e.g., September 2022 - Present)
            r"(?P<start_month>\w+)\s+(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_year>current|Current|present|Present)",

            # MMM YYYY - Present (e.g., Sep 2022 - Present)
            r"(?P<start_month>[a-zA-Z]{3})\s+(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_year>current|Current|present|Present)",

            # MM/YYYY - Present (e.g., 09/2022 - Present)
            r"(?P<start_month>\d{2})\/(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_year>current|Current|present|Present)",

            # YYYY.MM - YYYY.MM (e.g., 2022.09 - 2023.12)
            r"(?P<start_year>\d{4})\.(?P<start_month>\d{2})\s*[-–to]+\s*(?P<end_year>\d{4})\.(?P<end_month>\d{2}|current|Current|present|Present)",

            # YYYY.MM.DD - YYYY.MM.DD (e.g., 2022.09.01 - 2023.12.31)
            r"(?P<start_year>\d{4})\.(?P<start_month>\d{2})\.(?P<start_day>\d{2})\s*[-–to]+\s*(?P<end_year>\d{4})\.(?P<end_month>\d{2})\.(?P<end_day>\d{2}|current|Current|present|Present)",

            # DD/MM/YYYY - DD/MM/YYYY (e.g., 01/09/2022 - 31/12/2023)
            r"(?P<start_day>\d{2})\/(?P<start_month>\d{2})\/(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_day>\d{2})\/(?P<end_month>\d{2})\/(?P<end_year>\d{4}|current|Current|present|Present)",

            # MM-YY - MM-YY (e.g., 09-22 - 12-23)
            r"(?P<start_month>\d{2})-(?P<start_year>\d{2})\s*[-–to]+\s*(?P<end_month>\d{2})-(?P<end_year>\d{2}|current|Current|present|Present)",

            # DD-MM-YYYY - Present (e.g., 01-09-2022 - Present)
            r"(?P<start_day>\d{2})-(?P<start_month>\d{2})-(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_year>current|Current|present|Present)",

            # MM.DD.YYYY - MM.DD.YYYY (e.g., 09.01.2022 - 12.31.2023)
            r"(?P<start_month>\d{2})\.(?P<start_day>\d{2})\.(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_month>\d{2})\.(?P<end_day>\d{2})\.(?P<end_year>\d{4}|current|Current|present|Present)",
            r"(?P<start_month>[a-zA-Z]{3})\s+(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_month>[a-zA-Z]{3})\s+(?P<end_year>\d{4}|current|Current|present|Present)",
            r"(?P<start_month>\w+)\s+(?P<start_year>\d{4})\s*[-–to]+\s*(?P<end_month>\w+)\s+(?P<end_year>\d{4}|current|Current|present|Present)",
        ]

        experience_in_months = 0
        current_date = datetime.now()

        # Isolate the Experience section
        experience_section = self.extract_experience_section()

        employment_periods = []

        # Apply date patterns to the Experience section only
        for pattern in date_patterns:
            matches = re.finditer(pattern, experience_section, re.IGNORECASE)
            for match in matches:
                start_month = match.group("start_month")
                start_year = match.group("start_year")
                end_month = (
                    match.group("end_month")
                    if "end_month" in match.groupdict()
                    else None
                )
                end_year = (
                    match.group("end_year") if "end_year" in match.groupdict() else None
                )

                if end_year and end_year.lower() in ["present", "now"]:
                    end_date = current_date
                else:
                    # Construct date strings for parsing
                    start_date_str = f"{start_month} {start_year}"
                    end_date_str = (
                        f"{end_month} {end_year}" if end_month else f"{end_year}"
                    )

                    # Parse start and end dates
                    start_date = parse(start_date_str)
                    end_date = parse(end_date_str) if end_date_str else current_date

                if start_date and end_date and start_date <= end_date:
                    employment_periods.append((start_date, end_date))

        # Remove overlaps in periods
        employment_periods = self.remove_overlaps(employment_periods)

        # Calculate total experience in months
        for start_date, end_date in employment_periods:
            months = (end_date.year - start_date.year) * 12 + (
                end_date.month - start_date.month
            )
            experience_in_months += months

        # Convert months to years
        total_experience_years = (
            round(experience_in_months / 12, 2) if experience_in_months > 0 else None
        )
        return experience_in_months

    def extract_experience_section(self):
        """
        Helper function to extract the 'Experience' section from the resume text.
        """
        # Define the section headers to locate 'Experience' section and ignore others
        section_headers = [
            "EXPERIENCE",
            "WORK EXPERIENCE",
            "PROFESSIONAL EXPERIENCE",
            "CAREER HISTORY",
            "EMPLOYMENT HISTORY",
            "PROFESSIONAL BACKGROUND",
            "CAREER SUMMARY",
            "WORK HISTORY",
            "EMPLOYMENT EXPERIENCE",
        ]
        section_pattern = re.compile(
            r"(" + "|".join(map(re.escape, section_headers)) + ")", re.IGNORECASE
        )
        next_section_pattern = re.compile(
            r"(education|projects|skills|certifications|summary)", re.IGNORECASE
        )
        experience_pattern = re.compile(r"(Experience|Work History)", re.IGNORECASE)
        education_pattern = re.compile(
            r"(Education|Academic|Aualification)", re.IGNORECASE
        )
        project_pattern = re.compile(r"(Projects|Personal Projects)", re.IGNORECASE)

        experience_match = section_pattern.search(self.text)
        if not experience_match:
            return ""

        # Find the end of the Experience section by looking for next section (Education, Projects)
        start_pos = experience_match.end()

        next_section_match = next_section_pattern.search(self.text, start_pos)

        end_pos = next_section_match.start() if next_section_match else len(self.text)

        # Extract only the Experience section text
        experience_section = self.text[start_pos:end_pos]
        return experience_section

    def remove_overlaps(self, periods):
        """
        Removes overlapping periods from the list of employment periods.
        """
        # Sort periods by start date
        periods.sort(key=lambda x: x[0])
        merged_periods = []
        for current_start, current_end in periods:
            if not merged_periods:
                merged_periods.append((current_start, current_end))
            else:
                last_start, last_end = merged_periods[-1]
                if current_start <= last_end:
                    # Overlapping periods, merge them
                    merged_periods[-1] = (last_start, max(last_end, current_end))
                else:
                    merged_periods.append((current_start, current_end))
        return merged_periods

    def get_number_of_pages(self):
        """
        Returns the number of pages in the resume.
        """
        # For simplicity, we'll assume one page per 500 words
        word_count = len(self.text.split())
        pages = word_count // 500 + 1
        return pages

    def extract_hobbies(self):
        """
        Extracts hobbies if mentioned.
        """
        hobby_pattern = r"Hobbies|Interests|Activities"
        if re.search(hobby_pattern, self.text, re.IGNORECASE):
            # Extract hobbies mentioned after the keyword
            hobbies_text = self.text.split(
                re.search(hobby_pattern, self.text, re.IGNORECASE).group()
            )[-1]
            hobbies_list = hobbies_text.strip().split("\n")[0]
            return hobbies_list.strip()
        return None

    def get_extracted_data(self):
        """
        Returns the extracted data.
        """
        self.parse()
        return self.data



import os
from typing import Optional
import fitz
from docx import Document
import logging

logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
logger = logging.getLogger(__name__)

class FileParser:
    SUPPOERTED_TYPES = ('.pdf', '.docx', '.txt')
    
    def __init__(self, file_path: str) -> None:
        self.file_path = file_path
        self.file_type = self._determine_file_type()
        self.content: Optional[str] = None
    
    def _determine_file_type(self) -> str:
        _, ext = os.path.splitext(self.file_path.lower())
        if ext in self.SUPPOERTED_TYPES:
            return ext[1:]
        else:
            raise ValueError(f"Unsupported file type: {ext}")
    
    def parse(self) -> Optional[str]:
        parser_method = getattr(self, f'_parse_{self.file_type}', None)
        if not parser_method:
            raise NotImplementedError(f"Parser for {self.file_type} files is not implemented")
        
        try:
            self.content = parser_method()
            logger.info(f"Successfully parsed {self.file_path}")
            return self.content
        except Exception as e:
            logger.error(f"Error parsing {self.file_path}: {e}")
            return None
    
    def _parse_pdf(self) -> str:
        text = []
        try:
            with fitz.open(self.file_path) as doc:
                for page_num, page in enumerate(doc, start=1):
                    page_text = page.get_text()
                    text.append(page_text)
                    if page_num % 10 == 0:
                        logger.debug(f"Parsed {page_num} pages.")
            return '\n'.join(text)
        except Exception as e:
            logger.exception("Failed to parse PDF.")
            raise RuntimeError(f"Failed to parse PDF: {e}")
    
    def _parse_docx(self) -> str:
        try:
            doc = Document(self.file_path)
            full_text = [para.text for para in doc.paragraphs]
            return '\n'.join(full_text)
        except Exception as e:
            logger.exception("Failed to parse DOCX.")
            raise RuntimeError(f"Failed to parse DOCX: {e}")
    
    def _parse_txt(self) -> str:
        try:
            with open(self.file_path, 'r', encoding='utf-8') as f:
                return f.read()
        except UnicodeDecodeError:
            try:
                with open(self.file_path, 'r', encoding='latin-1') as f:
                    return f.read()
            except Exception as e:
                logger.exception("Failed to parse TXT with alternative encoding.")
                raise RuntimeError(f"Failed to parse TXT: {e}")
        except Exception as e:
            logger.exception("Failed to parse TXT.")
            raise RuntimeError(f"Failed to parse TXT: {e}")


import os
import unittest
from UniParse import FileParser

class TestFileParser(unittest.TestCase):
    def setUp(self):
        self.current_dir = os.path.dirname(__file__)
        self.txt_file = os.path.join(self.current_dir, 'sample.txt')
        self.pdf_file = os.path.join(self.current_dir, 'sample.pdf')
        self.docx_file = os.path.join(self.current_dir, 'sample.docx')

    def test_txt_parser(self):
        parser = FileParser(self.txt_file)
        content = parser.parse()
        self.assertIsNotNone(content)

    def test_pdf_parser(self):
        parser = FileParser(self.pdf_file)
        content = parser.parse()
        self.assertIsNotNone(content)

    def test_docx_parser(self):
        parser = FileParser(self.docx_file)
        content = parser.parse()
        self.assertIsNotNone(content)

if __name__ == '__main__':
    unittest.main()


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="UniParse",
    version="1.0.1",
    author="Hridesh",
    keywords='parse parser pdf docx txt uniparse uniparser',
    author_email="hridesh.khandal@gmail.com",
    description="A library to parse PDF, DOCX, and TXT files",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/hridesh-net/praserlib.git",
    packages=setuptools.find_packages(),
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Text Processing :: Linguistic',
    ],
    python_requires='>=3.8',
    install_requires=[
        "PyMuPDF>=1.18.0",
        "python-docx>=0.8.10",
        "nltk>=3.5",
        "spacy>=3.0.0",
        "dateparser>=1.2.0",
    ]
)


import re
import nltk
from .parser import FileParser


try:
    nltk.data.find('tokenizers/punkt')
    nltk.data.find('corpora/stopwords')
except LookupError:
    nltk.download('punkt_tab', quiet=True)
    nltk.download('stopwords', quiet=True)

class ResumeSummary:
    """
    A class to extract and summarize text from resumes.
    """

    def __init__(self, file_path):
        self.file_path = file_path
        self.text = ""
        self.summary = ""

    def extract_text(self):
        """
        Extracts text from the resume file using FileParser.
        """
        parser = FileParser(self.file_path)
        self.text = parser.parse()
        if not self.text:
            raise ValueError("Failed to extract text from the resume.")

    def preprocess_text(self):
        """
        Preprocesses the extracted text for summarization.
        """
        # Remove non-ASCII characters
        text = self.text.encode('ascii', 'ignore').decode()
        # Remove special characters and digits
        text = re.sub(r'[^a-zA-Z\s]', ' ', text)
        # Convert to lowercase
        text = text.lower()
        # Remove extra whitespace
        text = re.sub(r'\s+', ' ', text).strip()
        self.text = text

    def summarize(self, n=5):
        """
        Summarizes the preprocessed text into n sentences.
        """
        from nltk.tokenize import sent_tokenize, word_tokenize
        from nltk.corpus import stopwords
        from string import punctuation

        stop_words = set(stopwords.words('english') + list(punctuation))
        words = word_tokenize(self.text)

        # Calculate word frequencies
        freq = {}
        for word in words:
            if word not in stop_words:
                freq[word] = freq.get(word, 0) + 1

        # Normalize frequencies
        max_freq = max(freq.values()) if freq else 1
        for word in freq.keys():
            freq[word] = freq[word] / max_freq

        # Score sentences
        sentences = sent_tokenize(self.text)
        sentence_scores = {}
        for sentence in sentences:
            sentence_words = word_tokenize(sentence.lower())
            for word in sentence_words:
                if word in freq:
                    sentence_scores[sentence] = sentence_scores.get(sentence, 0) + freq[word]

        # Select top n sentences
        summarized_sentences = sorted(
            sentence_scores, key=sentence_scores.get, reverse=True
        )[:n]
        self.summary = ' '.join(summarized_sentences)

    def get_summary(self, n=5):
        """
        Executes the full pipeline to extract and summarize the resume text.
        """
        self.extract_text()
        self.preprocess_text()
        self.summarize(n=n)
        return self.summary